Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Tracks Functions">
%        <Item name="Track %top/%bottom Intensities" icon="Matlab">
%          <Command>MatlabXT::trackTopBotPercentPercentChange(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Track %top/%bottom Intensities">
%            <Command>MatlabXT::trackTopBotPercentPercentChange(%i)</Command>
%          </Item>
%        </SurpassComponent>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="Track %top/%bottom Intensities">
%            <Command>MatlabXT::trackTopBotPercentPercentChange(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
% 
%




function [] = trackTopBotPercentPercentChange(aImarisApplicationID)
%UNTITLED3 Summary of this function goes here
%   Detailed explanation goes here


if ~isnumeric(aImarisApplicationID)
    aImarisApplicationID=str2num(aImarisApplicationID);
end

if ~isnumeric(aImarisApplicationID) && ~ischar(aImarisApplicationID)
    aImarisApplicationID=0;
end
global vImarisApplication;

if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
   vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

vDataSet = vImarisApplication.GetDataSet.Clone;
vSurpassScene = vImarisApplication.GetSurpassScene;
aType = Imaris.tType.eTypeFloat;

if strcmp(vDataSet.GetType,'eTypeUInt8')
    vImarisApplication.GetDataSet.SetType(aType);
elseif strcmp(vDataSet.GetType,'eTypeUInt16')
    vImarisApplication.GetDataSet.SetType(aType);
end    
% vDataSet = vImarisApplication.GetDataSet.Clone;



%Get DataSet Info for voxel sizing etc...
vDataMin = [vDataSet.GetExtendMinX, vDataSet.GetExtendMinY, vDataSet.GetExtendMinZ];
vDataMax = [vDataSet.GetExtendMaxX, vDataSet.GetExtendMaxY, vDataSet.GetExtendMaxZ];
vDataSize = [vDataSet.GetSizeX, vDataSet.GetSizeY, vDataSet.GetSizeZ];

vTimeMax = vDataSet.GetSizeT - 1;

SizeX = vDataSet.GetSizeX;
SizeY = vDataSet.GetSizeY;
SizeZ = vDataSet.GetSizeZ;
PixelSize = (vDataSet.GetExtendMaxX)/SizeX;
PixelDepth = (vDataSet.GetExtendMaxZ)/SizeZ;
vNumberOfChannels = vDataSet.GetSizeC;


%Get Selected Surface, or ask user to choose a surface object from list
vDataItem = vImarisApplication.GetSurpassSelection;
nSurfaces=0;
surfSelected=0;
if ~isempty(vDataItem)  
    if ~vImarisApplication.GetFactory.IsSurfaces(vDataItem)
        vDataItem=[];
    end
end    
DialogTitle='Please choose a surface object';
if isempty(vDataItem)  
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if vImarisApplication.GetFactory.IsSurfaces(vDataItem)
            nSurfaces=nSurfaces+1;
            vSurfaces{nSurfaces} = vImarisApplication.GetFactory.ToSurfaces(vDataItem);    
            SurfNames{nSurfaces}=char(vSurfaces{nSurfaces}.GetName);
        end
    end    
else
    if vImarisApplication.GetFactory.IsSurfaces(vDataItem)
        vSurface=vImarisApplication.GetFactory.ToSurfaces(vDataItem);
        SurfName{1}=char(vSurface.GetName);
        nSurfaces=1;
        surfSelected=1;
    end     
end    
if surfSelected==0
  choice = menu(DialogTitle,SurfNames);
  vSurface = vSurfaces{choice};
end


%get selected Indices
selectedIndices = vSurface.GetSelectedIndices;
trackEdges = vSurface.GetTrackEdges;

beforeList=trackEdges(:,1);
afterList =trackEdges(:,2);


iter=0;
for i=1:numel(beforeList)
    if any(beforeList(i)==selectedIndices)
        iter=iter+1;
        beforeListKeepers(iter)=beforeList(i);
        afterListKeepers(iter)=afterList(i);
    end
end    

if isempty(selectedIndices)
    error('please choose a track to run this extension');
end

nTracks = numel(selectedIndices)-numel(beforeListKeepers);
%get time index for each surface object
for i=1:numel(selectedIndices)
    SurfaceTime(i) = vSurface.GetTimeIndex(selectedIndices(i));
end    
%create list of used (1) and unused object IDs (0)
% usedIndex = zeros(size(selectedIndices));
%find the object that is earliest amongst all of the objects for nTrack
%times


surfaceTimeEdit=SurfaceTime;
totalIters=0;
for i=1:nTracks
    objID=find(surfaceTimeEdit==min(surfaceTimeEdit));
    if numel(objID)>1
        objID=objID(1);
    end
    iter=0;
    doneHere=false;
    while doneHere==false;
        iter=iter+1;
        totalIters=totalIters+1;
        if totalIters==numel(selectedIndices)
            stopHereToDebug=3;
        end    
        surfIDtrack{i}(iter)=selectedIndices(objID);
        surfaceTimeEdit(objID)=Inf;
        listIdx = find(beforeListKeepers==selectedIndices(objID));
        if isempty(listIdx)
            doneHere=true;
        else    
            objID=find(selectedIndices==afterListKeepers(listIdx));
            if isempty(objID)
                doneHere=true;
            end    
        end    
    end
end    
msg1=[int2str(numel(selectedIndices)) ' surfaces found, belonging to ' int2str(nTracks) ' tracks. '];
msg2=['Please enter the percentile range of intensities you wish to analyze....i.e.(enter 13 for top/bottom 13 percent)'];

returnString = inputdlg([msg1 msg2]);
percentValue = str2num(returnString{1});

mtitle = ['do you want the TOP ' returnString '% , or the BOTTOM ' returnString '%  ?'];
choice1 = menu(mtitle,'TOP','BOTTOM');
if vDataSet.GetSizeC>1
    nChannels = vDataSet.GetSizeC;
    for jj=1:nChannels
        channelsList{jj}=int2str(jj);
    end
    choice2 = menu('choose channel of interest',channelsList);
    choice2 = choice2-1;
else
    choice2=0;
end    
vAllStatistics = vSurface.GetStatistics;
vNames       = cell(vAllStatistics.mNames);
vValues      = vAllStatistics.mValues;
vIds         = vAllStatistics.mIds;

%need to get Ellipsoid Axis A X...A Y...A Z, BX BY BZ,  CX,CY,CZ
%along with 
% Ellipsoid Axis A X
% Ellipsoid Axis A Y
% Ellipsoid Axis A Z
% 
% Ellipsoid Axis B X
% Ellipsoid Axis B Y
% Ellipsoid Axis B Z
% 
% Ellipsoid Axis C X
% Ellipsoid Axis C Y
% Ellipsoid Axis C Z
% 
% Ellipsoid Axis Length A
% Ellipsoid Axis Length B
% Ellipsoid Axis Length C
% 
% Position X
% Position Y
% Position Z
% 
% "Ellipsoid Axis A ..." defines one of the three principal axes of the surface.
% "Ellipsoid Axis B ..." and "Ellipsoid Axis C ..." the other two.
% 
% The three "Ellipsoid Axis Length ..." statistics contain the length of the corresponding axes.
% 
% To build the bounding box you need to calculate the corner positions:
% 
% For every dimension XYZ:
% Corner1 = "Cell Position" - "Ellipsoid Axis A" * "Ellipsoid Axis Length A" - "Ellipsoid Axis B" * "Ellipsoid Axis Length B" - "Ellipsoid Axis C" * "Ellipsoid Axis Length C"
% Corner2 = "Cell Position" - "Ellipsoid Axis A" * "Ellipsoid Axis Length A" - "Ellipsoid Axis B" * "Ellipsoid Axis Length B" + "Ellipsoid Axis C" * "Ellipsoid Axis Length C"
% 
% Replacing the minus signs by plus signs yields the other corners.

% for i=1:numel(vNames)
%     isBBAAlengthX(i) = strcmp('BoundingBoxAA Length X',vNames{i});
%     isBBAAlengthY(i) = strcmp('BoundingBoxAA Length Y',vNames{i});
%     isBBAAlengthZ(i) = strcmp('BoundingBoxAA Length Z',vNames{i});
% end    
% maxLengthX = max(vValues(isBBAAlengthX==1));
% maxLengthY = max(vValues(isBBAAlengthY==1));
% maxLengthZ = max(vValues(isBBAAlengthZ==1));
% 
% maxPixX = floor(maxLengthX/PixelSize)
% maxPixY = floor(maxLengthY/PixelSize)
% maxPixZ = floor(maxLengthZ/PixelDepth)




for i=1:numel(vNames)
    isPosX(i)=strcmp('Position X',vNames{i});
    isPosY(i)=strcmp('Position Y',vNames{i});
    isPosZ(i)=strcmp('Position Z',vNames{i});
    
    isEAAX(i)=strcmp('Ellipsoid Axis A X',vNames{i});
    isEAAY(i)=strcmp('Ellipsoid Axis A Y',vNames{i});
    isEAAZ(i)=strcmp('Ellipsoid Axis A Z',vNames{i});
    
    isEABX(i)=strcmp('Ellipsoid Axis B X',vNames{i});
    isEABY(i)=strcmp('Ellipsoid Axis B Y',vNames{i});
    isEABZ(i)=strcmp('Ellipsoid Axis B Z',vNames{i});
    
    isEACX(i)=strcmp('Ellipsoid Axis C X',vNames{i});
    isEACY(i)=strcmp('Ellipsoid Axis C Y',vNames{i});
    isEACZ(i)=strcmp('Ellipsoid Axis C Z',vNames{i});
    
    isEALA(i)=strcmp('Ellipsoid Axis Length A',vNames{i});
    isEALB(i)=strcmp('Ellipsoid Axis Length B',vNames{i});
    isEALC(i)=strcmp('Ellipsoid Axis Length C',vNames{i});
end


xLow=0;
yLow=0;
zLow=0;
xHigh=0;
yHigh=0;
zHigh=0;


for j=1:nTracks
    for i=1:numel(surfIDtrack{j})
        surfID=surfIDtrack{j}(i);
        
        idPosX = isPosX' & vIds==surfID;
        locPosX = find(idPosX==1);
        posX = vValues(locPosX);
        %
        idPosY = isPosY' & vIds==surfID;
        locPosY = find(idPosY==1);
        posY = vValues(locPosY);
        %
        idPosZ = isPosZ' & vIds==surfID;
        locPosZ = find(idPosZ==1);
        posZ = vValues(locPosZ);
        %
        idEAAX = isEAAX' & vIds==surfID;
        locEAAX = find(idEAAX==1);
        EAAX = vValues(locEAAX);
        %
        idEAAY = isEAAY' & vIds==surfID;
        locEAAY = find(idEAAY==1);
        EAAY = vValues(locEAAY);
        %
        idEAAZ = isEAAZ' & vIds==surfID;
        locEAAZ = find(idEAAZ==1);
        EAAZ = vValues(locEAAZ);
        %
        idEABX = isEABX' & vIds==surfID;
        locEABX = find(idEABX==1);
        EABX = vValues(locEABX);
        %
        idEABY = isEABY' & vIds==surfID;
        locEABY = find(idEABY==1);
        EABY = vValues(locEABY);
        %
        idEABZ = isEABZ' & vIds==surfID;
        locEABZ = find(idEABZ==1);
        EABZ = vValues(locEABZ);
        %
        idEACX = isEACX' & vIds==surfID;
        locEACX = find(idEACX==1);
        EACX = vValues(locEACX);
        
        idEACY = isEACY' & vIds==surfID;
        locEACY = find(idEACY==1);
        EACY = vValues(locEACY);
        
        idEACZ = isEACZ' & vIds==surfID;
        locEACZ = find(idEACZ==1);
        EACZ = vValues(locEACZ);
        
        idEALA = isEALA' & vIds==surfID;
        locEALA = find(idEALA==1);
        EALA = vValues(locEALA);
        
        idEALB = isEALB' & vIds==surfID;
        locEALB = find(idEALB==1);
        EALB = vValues(locEALB);
        
        idEALC = isEALC' & vIds==surfID;
        locEALC = find(idEALC==1);
        EALC = vValues(locEALC);
        
        pXYZ=[posX posY posZ];
        Aaxis = EALA * [EAAX EAAY EAAZ];
        Baxis = EALB * [EABX EABY EABZ];
        Caxis = EALC * [EACX EACY EACZ];
        
        
        corner1=pXYZ - Aaxis - Baxis - Caxis;
        corner2=pXYZ - Aaxis - Baxis + Caxis;
        corner3=pXYZ + Aaxis + Baxis + Caxis;
        corner4=pXYZ + Aaxis + Baxis - Caxis;
        minXYZ1 = min(min(corner1,corner2),min(corner3,corner4));
        maxXYZ1 = max(max(corner1,corner2),max(corner3,corner4));
        
        corner5=pXYZ - Aaxis + Baxis - Caxis;
        corner6=pXYZ + Aaxis - Baxis - Caxis;
        corner7=pXYZ - Aaxis - Baxis - Caxis;
        corner8=pXYZ + Aaxis - Baxis + Caxis;
        
        minXYZ2= min(min(corner5,corner6),min(corner7,corner8));
        maxXYZ2= max(max(corner5,corner6),max(corner7,corner8));
        
        minXYZ = min(minXYZ1,minXYZ2);
        maxXYZ = max(maxXYZ1,maxXYZ2);
        
        minXYZ = minXYZ - vDataMin;
        maxXYZ = maxXYZ - vDataMin;
%         minXYZ = corner1;
        
        if minXYZ(1)<vDataMin(1)
%             display(['x value is too low: X = ' int2str(minXYZ(1)) '  -  surfID = ' int2str(surfID)]);
            xLow=xLow+1;
            minXYZ(1)=vDataMin(1);
        end
        if minXYZ(2)<vDataMin(2)
%             display(['y value is too low: Y = ' int2str(minXYZ(2)) '  -  surfID = ' int2str(surfID)]);
            yLow=yLow+1;
            minXYZ(2)=vDataMin(2);
        end
        if minXYZ(3)<vDataMin(3)
%             display(['z value is too low: Z = ' int2str(minXYZ(3)) '  -  surfID = ' int2str(surfID)]);
            zLow=zLow+1;
            minXYZ(3)=vDataMin(3);
        end    
        if maxXYZ(1)>vDataMax(1)
%             display(['x value is too High: X = ' int2str(maxXYZ(1)) '  -  surfID = ' int2str(surfID)]);
            xHigh=xHigh+1;
            maxXYZ(1)=vDataMax(1);
        end
        if maxXYZ(2)>vDataMax(2)
%             display(['y value is too High: Y = ' int2str(maxXYZ(2)) '  -  surfID = ' int2str(surfID)]);
            yHigh=yHigh+1;
            maxXYZ(2)=vDataMax(2);
        end
        if maxXYZ(3)>vDataMax(3)
%             display(['z value is too High: Z = ' int2str(maxXYZ(3)) '  -  surfID = ' int2str(surfID)]);
            zHigh=zHigh+1;
            maxXYZ(3)=vDataMax(3);
        end    
        
        
        newX = minXYZ(1);
        newY = minXYZ(2);
        newZ = minXYZ(3);
        newXpix = floor(newX/PixelSize);
        newYpix = floor(newY/PixelSize);
        newZpix = floor(newZ/PixelDepth);
        
        if newXpix<0
            newXpix=0;
        end
        if newYpix<0
            newYpix=0;
        end
        if newZpix<0
            newZpix=0;
        end
        
        
        maxX = floor((maxXYZ(1))/PixelSize);
        maxY = floor((maxXYZ(2))/PixelSize);
        maxZ = floor((maxXYZ(3))/PixelDepth);
        lenX = maxX - newXpix;
        lenY = maxY - newYpix;
        lenZ = maxZ - newZpix;
        if newXpix==0
            lenX=lenX+1;
        end
        if newYpix==0
            lenY=lenY+1;
        end
        if newZpix==0
            lenZ=lenZ+1;
        end    
        if (lenX+newXpix)>vDataSize(1)
            lenX = vDataSize(1)-newXpix;
        end
        if (lenY+newYpix)>vDataSize(2)
            lenY = vDataSize(2)-newYpix;
        end
        if (lenZ + newZpix)>vDataSize(3)
            lenZ = vDataSize(3)-newZpix;
        end    
        currentTimePoint = vSurface.GetTimeIndex(surfID)
        display(surfID)
        thisSubVolume{j}{i} = vDataSet.GetDataSubVolumeFloats(uint8(newXpix),uint8(newYpix),uint8(newZpix),choice2,vSurface.GetTimeIndex(surfID),uint8(lenX),uint8(lenY),uint8(lenZ));
    
        theseVals = reshape(thisSubVolume{j}{i},numel(thisSubVolume{j}{i}),1);
        theseVals = theseVals(theseVals>0);
        if choice1==1 
            theseVals = sort(theseVals,'descend');
        end
        if choice1==2
            theseVals = sort(theseVals,'ascend');
        end    
        len2keep = round(percentValue*.01*numel(theseVals));
        data{j}(i) = mean(theseVals(1:len2keep));
        dataOut(j,i) = mean(theseVals(1:len2keep));
    end
end    
        
write2path = uiputfile('*.xls','where do you want to save the output data?');
display(['writing..... ' write2path]);
xlswrite(write2path,dataOut);
display(['file written to : ' write2path]);


end
